const path = require('path');
const rollup = require('rollup');
const nodeEval = require('node-eval');
const  buble = require('@rollup/plugin-buble')
const  commonjs = require('@rollup/plugin-commonjs')
const  replace = require('@rollup/plugin-replace')
const  jsonlint = require('rollup-plugin-jsonlint') // 需要手工安装依赖 @rollup/plugin-json
const  es6template = require('rollup-plugin-es6template')
const  { string } = require('rollup-plugin-string')
const {
  done,
  info,
  warn,
  error
} = require('@vue/cli-shared-utils')
const builtinModules = require('builtin-modules');

module.exports=function bundleRun(api,input,runPath) {
  const { dependencies } = require(api.resolve('./package.json'))
  const external = Object.keys(dependencies || {}).concat(builtinModules);
  info('Start Bundling express app...');
  return rollup.rollup({
      input,
      external,
      // 没有用的代码去掉，节约空间
      treeshake: {
          moduleSideEffects: false
      },
      onwarn (warning, warn) {
        // skip certain warnings
        if (warning.code === 'UNUSED_EXTERNAL_IMPORT') return;
        // throw on others
        if (warning.code === 'NON_EXISTENT_EXPORT') throw new Error(warning.message);
        // Use default for everything else
        warn(warning);
      },
      plugins: [
          commonjs(),
          es6template({
              include: ['**/*.html', '**/*.css']
          }),
          string({
              // Required to be specified
              include: ['**/*.txt', '**/*.pem']
          }),
          // 允许json中包含注释
          jsonlint({ mode: 'json5' }),
          // 自动安装依赖
          // auto(),
          replace({
              ENV: JSON.stringify(process.env.NODE_ENV || 'development')
          }),
          buble({
              exclude: 'node_modules/**',
              transforms: { asyncAwait: false ,dangerousForOf: true}
          })
      ],
  }).then(bundle=>{
    return bundle.generate({
        name:'Server',
        dir: path.resolve('./dist'),
        format: 'cjs',
        exports: 'named', // 保留exports，不自动加default
        compact: false,
        preferConst: true,
        inlineDynamicImports:true,
    });
  }).then(({output})=>{
      try{
        let srv=nodeEval(output[0].code,runPath);
        if(srv) {
          done('Bundled success!')
          return Promise.resolve(srv.default  || srv);
        }else{
          return Promise.reject('Can not return any function from bundle');
        }
      }catch(err){
        error(err);
        return Promise.reject('Bundle failed!');
      }
  });
}