import path from 'path'
import builtinModules from 'builtin-modules'
import buble from '@rollup/plugin-buble'
import commonjs from '@rollup/plugin-commonjs'
import replace from '@rollup/plugin-replace'
import jsonlint from 'rollup-plugin-jsonlint' // 需要手工安装依赖 @rollup/plugin-json
import generatePackageJson from 'rollup-plugin-generate-package-json'
import es6template from 'rollup-plugin-es6template'
import { string } from 'rollup-plugin-string'
import { terser } from 'rollup-plugin-terser'

const {
  info,
  done,
  warn,
  error
} = require('@vue/cli-shared-utils')
export default {
  input: './src/index.js',
  external:builtinModules,
  output: {
    dir: path.resolve('./dist'),
    format: 'cjs',
    entryFileNames: 'index.js',
    chunkFileNames: 'libs/[name].js',//动态import()的内容打包到这里
    exports: 'named', // 保留exports，不自动加default
    compact: false,
    preferConst: true,
    plugins: [
      terser() // 压缩混淆代码
    ]
  },
  // 没有用的代码去掉，节约空间
  treeshake: {
    moduleSideEffects: false
  },
  onwarn (warning, warn) {
    // skip certain warnings
    if (warning.code === 'UNUSED_EXTERNAL_IMPORT') return;
    // throw on others
    if (warning.code === 'NON_EXISTENT_EXPORT') error(warning.message);
    // Use default for everything else
    warn(warning);
  },
  plugins: [
    commonjs(),
    //在dist目录创建package.json文件，自动包含使用的依赖
    generatePackageJson({
      baseContents: (pkg) => ({
        name: pkg.name,
        author: pkg.author,
        license: pkg.license,
        main: 'index.js',
        dependencies: {}
      })
    }),
    // 用于快速格式化模板
    es6template({
      include: ['**/*.html', '**/*.htm']
    }),
    // 直接把内容包含进来
    string({
      include: ['**/*.pem','**/*.txt']
    }),
    // 允许json中包含注释
    jsonlint({ mode: 'json5' }),
    buble({
      transforms: {
        asyncAwait: false,
        dangerousForOf: true,
      },
      exclude: /node_modules/
    }),
    replace({
      ENV: JSON.stringify(process.env.NODE_ENV || 'development')
    }),
    notice()
  ],
}

function notice(){
  return  {
    name: 'notice',
    buildStart() {
      info('Start building ...');
    },
    writeBundle() {
      done('Build completed!')
      warn('Don\'t forget to run "npm install" in dist folder before run it');
    }
  };
}
